﻿// Written in the D programming language, version 2.x
/**The matrix module provides the template Matrix type as well as exceptions thrown.
 * License: Lastest LGPL
 * Authors: Pedro Ferreira, silverclaw@users.sourceforge.net*/
module matrix;

version(Phobos) private import std.conv: to; //to convert stuff around
version(Phobos) private import std.contracts:assumeUnique; //is this used at all yet?
//                                             BUG: isUnsigned is not found, even though it is documented
version(Phobos) private import std.traits: isArray, unsigned; //for the opShl and opCat
version(Phobos) private import std.math: approxEqual, conj, abs; //to compare floating point numbers, to conjugate img numbrs

///This is the base class for all exceptions thrown by the matrix class
public class MatrixException : Exception
{
	public this(in string msg)
	{
		super(msg);
	}
}

///This Exception is thrown when an operation is requested and the matrix's size makes such operation impossible
public class MatrixSizeException : MatrixException
{
	public this(in string msg)
	{
		super(msg);
	}
}

/**The Matrix class: instantiate with Matrix!(type).$(BR)
 * It can be used has a vector, by declaring it with a single row.
 * Remarks: the first element of a _Matrix is matrix[0,0] and the last is matrix[m-1,n-1]
 * on a mxn _Matrix.$(BR)
 * If you instanciate a _Matrix with a non-numeric type, certain functions may
 * behave erraticaly, fail to run or even to compile.
 * Examples:
 * ---------------------
 * auto matrix=new Matrix!(int)(3,3); //create a 3x3 matrix
 * auto another=new Matrix!(ulong)(3, 3); //same as above,for a different typy
 * auto third=matrix*another; //create a third matrix by multiplying the previous 2
 * delete matrix; //deleting a matrix will delete it's contents as well
 * ---------------------*/
public class Matrix(Type) //I would like to use (Type=real) if possible...
{
	/**Creates a new Matrix of size ($(I rows) x $(I columns)) or degree $(I degree) and fills it using $(I fill).
	 *
	 * $(B fill) may be one of the following: $(BR)
	 * &nbsp;* (none) this will cause the matrix to be initialized with Type's initializer;$(BR)
	 * &nbsp;* a constant value (of the same type of the Matrix);$(BR)
	 * &nbsp;* a function returning a value of the type of the Matrix, this function is called for every element;$(BR)
	 * &nbsp;* an array containing the values of the Matrix;$(BR)
	 * &nbsp;* an array containing the values for every row of the Matrix (in this case,
	 * the number of columns is the size of the vector);$(BR)
	 * &nbsp;* an array of arrays. In this case, no size is specified (rows or columns).
	 *
	 * Note: The validity (non-nullity) of $(I fill) is only checked in debug version via "in contract".
	 * See_Also: fill(), dup()
	 * Throws: MatrixSizeException if requested matrix has size==0 or if the data supplied isn't enough
	 * Bugs: In the case of this(Type fill[][]) it is assumed that the size of the first 
	 * array is less or equal to the remaining vectors. This is only checked in the debug version.
	 * If this assumption is invalid, the program will either end with "Access Violation" (on Windows),
	 * "Segmentation Fault" (or SIGSEV on Unix) or behave erraticaly.
	 */
	public this(in size_t degree)
	{
		this(degree, degree);
	}
	
	///ditto
	public this(in size_t rows, in size_t columns)
	{
		if ((columns==0)||(rows==0)) throw new MatrixSizeException("Matrix minimum size is 1");
		data=new Type[][](rows,columns);
	}
	
	///ditto
	public this(in size_t rows, in size_t columns, in Type fill)
	{
		this(rows, columns);
		foreach(Type[] row; data)
			foreach(inout Type element; row)
				element=fill;
		/+DMD BUG:
		Altough it is documented at www.digitalmars.com/d/arrays.html under "Array Setting"
		neither of the following lines work: (at leat the first one should)
		* foreach(ref Type[] row; data) row=fill;
		* data[][]=fill;
		+/
	}

	///ditto
	public this(in size_t rows, in size_t columns, in Type function() fill)
	in
	{
		assert(fill !is null, "Matrix: function given to this() is null");
	}
	body
	{
		this(rows, columns);
		foreach(Type[] row; data)
			foreach(inout Type value; row)
				value=fill();
	}

	///ditto
	public this(in size_t rows, in size_t columns, in Type delegate() fill)
	in
	{
		assert(fill !is null, "Matrix: delegate given to this() is null");
	}
	body
	{
		this(rows, columns);
		foreach(Type[] row; data)
			foreach(inout Type element; row)
				element=fill();
	}

	///ditto
	public this(in size_t rows, in size_t columns, in Type fill[])
	in
	{
		assert(fill !is null, "Matrix: The fill buffer is null");
		assert((fill.length>=rows*columns), "Matrix: The fill buffer is too small for the requested matrix");
	}
	body
	{
		this(rows, columns);
		foreach(i, Type[] row; data)
			foreach(j, inout Type element; row)
				element=fill[i*columns+j];
	}

	///ditto
	public this(in size_t rows, in Type fill[])
	in
	{
		assert(fill !is null, "Matrix: The fill buffer is null");
	}
	body
	{
		foreach(Type[] row; this.data)
			foreach(j, inout Type element; row)
				element=fill[j];
	}

	///ditto
	public this(in Type fill[][])
	in
	{
		assert(fill !is null, "Matrix: data given to this() (fill) is null");
	}
	body
	{
		data=new Type[][](fill.length, fill[0].length);
		foreach(i,const Type[] row; fill)
		{
			assert(row.length==fill[0].length, "Matrix: Supplied buffer is not consistent.");
			foreach(j, element; row)
				data[i][j]=element;
		}
	}

	//DDOC Bug: DDOC does not put arguments in itallic for "ditto"'ed functions.
	/**Fills the Matrix with the supplied buffer (data), $(I value) or return of the function $(I func).
	 * If the buffer is of the size of a row, it is duplicated for every row in the Matrix.
	 * Note: The validity (non-nulity) of data and of the functions is only checked on the debug version.
	 * Throws: MatrixSizeException if the contents of the buffer are not conformant with this Matrix's size
	 * See_Also: dup()*/
	public void fill(in Type data[][])
	in
	{
		assert(data !is null, "Matrix: data given to fill() is null");
	}
	body
	{
		if (data.length<this.data.length) throw new MatrixSizeException("Insuficient data in buffer");
		foreach(const Type[] row; data) if (row.length<this.data[0].length) throw new MatrixSizeException("Insuficient data in buffer");
		foreach(i, Type[] row; this.data)
			foreach(j, inout element; row)
				element=data[i][j];
	}

	///ditto
	public void fill(in Type data[])
	in
	{
		assert(data !is null, "Matrix: data given to fill() is null");
	}
	body
	{
		if (data.length==this.n()) //single row vector
		{
			foreach(i, Type[] row; this.data)
				foreach(j, inout Type element; row)
					element=data[j];
			return;
		}
		if (data.length<=this.m()*this.n()) throw new MatrixSizeException("Insuficient data in buffer");
		//TODO: redo this using 2 for's and using this[i,j]
		foreach(i, Type[] row; this.data)
			foreach(j, inout Type element; row)
				element=data[i*n+j];
	}
	
	///ditto
	public void fill(in Type value)
	{
		foreach(i, Type[] row; this.data)
			foreach(j, inout Type element; row)
				element=value; //see the bug at this(c,r,fill)
	}
	
	///ditto
	public void fill(in Type function() func)
	in
	{
		assert(func !is null, "Matrix: function given to fill() is null");
	}
	body
	{
		foreach(i, Type[] row; this.data)
			foreach(j, inout Type element; row)
				element=func();
	}
	
	///ditto
	public void fill(in Type delegate() del)
	in
	{
		assert(del !is null, "Matrix: delegate given to fill() is null");
	}
	body
	{
		foreach(i, Type[] row; this.data)
			foreach(j, inout Type element; row)
				element=del();
	}

	/**Returns: a duplicate of this Matrix.
	 * Note: Instanciate it with a type to cast the Matrix.
	 * With Phobos, the elements are cast with std.conv.to!(T)() which may throw convertion errors.
	 * Examples:
	 * --------------------------
	 * auto matrix=new Matrix!(real)(4, 5, 5.3); //create a 4x5 matrix filled with "5.3"
	 * auto matrix2=new Matrix!(int)(4, 5, 5); //same as above for int
	 * assert(matrix.dup!(int)==matrix2); //this will evaluate to "true" (rounding of 5.3==5)
	 * auto another=matrix2.dup(); //simple duplication. "another" is now typed "Matrix!(int)"
	 * --------------------------*/
	public const Matrix!(T) dup(T=Type)()
	{
		static if (is(T==Type)) return new Matrix!(T)(data);
		else //DMD bug: with the "-w" option, this statement is not reachable
		{	//as such, I added an "else" clause to the static if, which sorted things out
			auto ret=new Matrix!(T)(this.m(), this.n());
			for (auto i=0; i<this.m(); i++)
				for (auto j=0; j<this.n(); j++)
					version(Phobos) ret[i,j]=to!(T)(this[i,j]);
					else static assert(false, "You must specify which library you are using (-version=Phobos)");
			return ret;
		}
	}

	/**Returns an identity Matrix (main diagonal=1, others=0)
	 * See_Also: setIdentity*/
	public static Matrix!(Type) identity(in size_t size)
	{
		Matrix!(Type) ret=new Matrix(size, size);
		ret.setIdentity();
		return ret;
	}

	///Sets this matrix to identity Matrix
	public void setIdentity()
	{
		foreach(i, Type[] row; data)
			foreach(j, inout Type value; row)
				if (i==j) value=cast(Type)1; //need to cast this to complex types
				else value=cast(Type)0; //ditto
	}

	/**Converts this Matrix to a string on the format [[value_1x1,...,value_1xn],...,[value_mx1,...,value_mxn]]
	 * Examples:
	 * ----------------------------
	 * auto test=new Matrix!(int)(3, 3, 9); //create a 3x3 matrix filled with 9
	 * write(test.toString());
	 * ----------------------------
	 * Output:
	 * [[9,9,9],[9,9,9],[9,9,9]]
	 * 
	 * Note: On Phobos, the function uses the std.conv.to!() function,
	 * which may throw convertion errors (to type string).*/
	override public const string toString()
	{
		version(Phobos) return (to!(string)(data));
	}

	/**Converts this Matrix to a string using seperator to seperate rows. $(BR)
	 * toString("; ") would create [row_1; row_2; ...; row_n] 
	 * Note: this function requires review.*/
	public const string toString(in string seperator)
	{
		//generate size for string. 2 for "[" "]" + (rows-1) for ";" + columns for 'seperator'
		char[] ret;
		scope(failure/*exit*/) delete ret; //does to!(string)(ret) duplicate ret? For other libraries, should this change?
		ret~='['; //TODO: preallocate buffer to speedup
		foreach(const Type[] row; data)
			foreach(j, const Type value; row)
			{
				version(Phobos) ret~=to!(char[])(value);
				else static assert(false, "You must specify which library you are using (-version=Phobos)");
				//TODO: Other versions add nothing....
				if (j==(row.length-1)) ret~=seperator;
				else ret~=' ';
			}
		ret[$-1]=']';
		version (Phobos) return to!(string)(ret);
		else static assert (false, "You must specify which library you are using (-version=Phobos)");
		/+version(Phobos) return assumeUnique(ret);
		else return ret.idup; //return an invariant copy+/
	}

	///Returns: a copy of the main diagonal of this Matrix.
	public const Type[] getDiagonal()
	{
		Type ret[]=new Type[]((this.m()>this.n())? this.n() : this.m());
		scope(failure) delete ret;
		for (auto i=0; i<ret.length; i++) ret[i]=data[i][i];
		return ret;
		/+version(Phobos) return assumeUnique(ret); //we generated this array
		else return ret.idup;+/
	}

	///Returns: this Matrix's transposed Matrix. This doesn't change the current Matrix.
	public const Matrix!(Type) getTranspose()
	{
		Matrix!(Type) ret=new Matrix!(Type)(this.n(), this.m());
		scope(failure) delete ret;
		for(auto i=0; i<this.m(); i++)
			for(auto j=0; j<this.n(); j++)
				ret[j,i]=this[i,j];
		return ret;
	}

	///Transposes this Matrix.
	public void transpose()
	{
		Type[][] newData=new Type[][](this.n(), this.m()); //generate new data
		scope(failure) delete newData;
		for(auto i=0; i<this.m(); i++)
			for(auto j=0; j<this.n(); j++)
				newData[j][i]=this[i,j];
		delete data; //delete previous data
		data=newData; //reassign data
		return;
	}

	/**Swaps two rows
	 * Throws: ArrayBoundsError.*/
	public void swapRow(in size_t row1, in size_t row2)
	{
	    Type[] row=data[row1];
	    data[row1]=data[row2];
	    data[row2]=row;
	}

	/**Swaps two columns
	 * Throws: ArrayBoundsError.*/
	public void swapCol(in size_t col1, in size_t col2)
	{
	    Type value;
	    for(auto i=0; i<m; i++) //all rows
	    {
            value=this[i, col1];
            this[i, col1]=this[i, col2];
            this[i, col2]=value;
	    }
	}
	
	/**Returns: this Matrix's data arrays 
	 * Note: To obtain a copy, use "matrix.getData.dup();"$(BR)
	 * The data arrays are assumed to be accessible only by the current instance of
	 * Matrix, as such, when this instance is deleted, it's data arrays are deleted
	 * as well.*/
	public Type[][] getData()
	{
		return data;
	}
	
	/**Returns: this Matrix's _determinant.
	 * Throws: MatrixSizeException if this matrix has size less than 2 or if this matrix is not square
	 *
	 * Note: The _determinant of a Matrix is calculated by:$(BR)
	 * Summation from i=0 to k of the value at [i,j] multiplied by it's cofactor and by the _determinant
	 * of the minor matrix, formed by excluding column i and row j. The cofactor is -1 if (i+j) is even
	 * and 1 otherwise. The initial j would be 0 and is increased by 1 every new minor matrix.$(BR)
	 * It may be faster to .reduce() the matrix and multiplying the values on it's main diagonal.$(BR)
	 * You may instanciate the function with a specific return type. If not, it will default to the type
	 * with which the Matrix is instanciated. Note that even though the final _determinant may be less
	 * than Type.max, it is possible that an overflow occurs during the calculation.$(BR)
	 * Even if the Matrix is instanciated to an unsigned type, it's determinant may be negative.
	 *
	 * Remarks: If two rows or columns on this matrix are equal, the _determinant will be 0.
	 * If any row or column is made of zeroes, the _determinant will be 0.$(BR)
	 * A Matrix with _determinant 0 is singular (non-invertible).
	 *
	 * See_Also: isSingular()*/
	public const T determinant(T=Type)()
	{
		if (this.m()!=this.n()) throw new MatrixSizeException("Cannot calculate the determinant of a non-square matrix");
		final k=this.m();
		if (k==2) return cast(T)(this[0,0]*this[1,1]-this[0,1]*this[1,0]); //straight determinant
		if (k==1) return cast(T)(this[0,0]); //single value matrix
		//initial determinant value
		T det=cast(T)(this[0,0]*this.minorMatrix(0,0).determinant!(T)());
		//multiply it by the other determinants
		bool cofactor=true; //first cofactor is -1
		for (auto i=1; i<k; i++)
		{
			if (cofactor) det-=this[i,0]*this.minorMatrix(i,0).determinant!(T)(); //cofactor: -1
			else det+=this[i,0]*this.minorMatrix(i,0).determinant!(T)(); //cofactor: 1
			cofactor=!cofactor;
		}
		return det;
	}
	
	/**Returns: this Matrix's minor Matrix obtained by excluding
	 * _row row and column col. The new Matrix's data is copied.
	 * Throws: MatrixSizeException if the current Matrix is too small
	 * or if the requested column or _row is not valid.*/
	public const Matrix!(Type) minorMatrix(in size_t row, in size_t col)
	{
		if (row>=this.m()||col>=this.n())
			throw new MatrixSizeException("Failed to create minorMatrix: requested row or column is invalid");
		Matrix!(Type) ret=new Matrix!(Type)(this.m()-1, this.n()-1);
		scope(failure) delete ret;
		size_t ret_c=0, ret_r=0;
		for (auto r=0; r<this.m(); r++)
		{
			if (r==row) continue; //do not copy this row
			for (auto c=0; c<this.n(); c++)
			{
				if (c==col) continue; //do not copy this column
				ret[ret_r,ret_c]=this[r,c];
				ret_c++;
			}
			ret_c=0;
			ret_r++;
		}
		return ret;
	}
	
	/**Returns: this Matrix's _trace, being the summation of the values on it's main diagonal.
	 * Note: You may specify a return type for this function. If not, it will default to the type
	 * of the instanciated Matrix. Caution is needed for large matrices to prevent overflow.
	 * Throws: MatrixSizeException if this Matrix is not square.*/
	public const T trace(T=Type)()
	{
		if (this.m()!=this.n()) throw new MatrixSizeException("Cannot calculate the trace of a non-square matrix");
		T result=this[0,0];
		for (auto k=1; k<this.m(); k++) result+=this[k,k];
		return result;
	}
	
	/**Reduces a Matrix to a triangle Matrix (values below the main diagonal set to 0)
	 * using the Gaussian Elimination algorithm. The Matrix will then be in row echelon form
	 * (not to be confused with "reduced row echelon form"), meaning that all values along the
	 * main diagonal are 1.
	 * Note: The Gaussian Elimination algorithm is known to be unstable for some matrices.*/
	public void reduce()
	{
		size_t i, j, maxi; //algorithm shamelessly taken from wikipedia.com
		while(i<this.m()&&j<this.n())
		{
			maxi=i;
			for (auto k=i+1; k<this.m(); k++)
			{
				version(Phobos)
				{	//since isUnsigned doesn't work, try this way around
					static if (is(unsigned!(Type)==Type)) {if (this[k,j]>this[maxi,j]) maxi=k;}
					else {if (abs(this[k,j])>abs(this[maxi,j])) maxi=k;}
				}
				else static assert(false);
			}
			if (!approxEqual(this[maxi,j], 0))
			{
				this.swapRow(i, maxi);
				foreach(ref element; this[i]) element/=this[i,j];
				for (auto u=i+1; u<this.m(); u++) this[u,j]=cast(Type)(this[u,j]-this[i,j]*this[u,j]);
				i++;
			}
			j++;
		}
	}
	
	///ditto
	public const Matrix getReduced()
	{
		auto ret=this.dup();
		scope(failure) delete ret;
		ret.reduce();
		return ret;
	}
	
	/**Solves a system of linear equations defined by this Matrix.
	 *
	 * Example:
	 * -------------------------------------
	 * auto eqSys=new Matrix!(int)(3, 4);
	 * eqSys[0]=[2,3,5,3]; //2a+3b+5c=3
	 * eqSys[1]=[1,-3,0,5]; //1a-3b+0c=5
	 * eqSys[2]=[2,3,1,9]; //2a+3b+1c=9
	 * int solution[]=eqSys.solve();
	 * write(solution); //will print: "[-57 5 9]" -> a=-57, b=5, c=9
	 * -------------------------------------
	 * Output: [-57 5 9]
	 *
	 * Returns: a single array of numbers representing the solution of the system. 
	 * You may specify the resolution of the calculation (type of return). This is useful
	 * for bigger equation systems.
	 * See_Also: reduce()
	 * Throws: MatrixSizeException if this Matrix is not of the form [m,m+1].*/
	public const T[] solve(T=Type)()
	{
		if (this.n()!=(this.m()+1)) throw new MatrixSizeException("Cannot solve the system: matrix size does not match [m,m-1]");
		T[] ret=new T[](this.m());
		scope(failure) delete ret;
		auto matrix=this.dup!(T)();
		matrix.reduce();
		T factor=0;
		//TODO something with the matrix to obtain the factors...
		foreach_reverse(i, ref element; ret)
		{
			element=0; //in case of float, the initial value is NaN...
			//for each element on the return vector (solution) calculate the factors first:
			factor=0;
			for (auto j=i; j<ret.length; j++) factor+=ret[j]*this[i, j];
			element=this[i, this.m()]-factor;
		}
		return ret;
	}

	///Returns a single one-dimention array with a copy of this Matrix's data
	public const Type[] getVector()
	{
		Type[] ret=new Type[](this.m()*this.n());
		scope(failure) delete ret;
		for (auto i=0; i<this.m(); i++)
			for (auto j=0; j<this.n(); j++)
				ret[i*n+j]=this[i, j];
		return ret;
	}
	
	/**Divides all values of this matrix by it's determinant.
	 * You can specify the precision of the calculation by instanciating the
	 * function with a specific type (int, real...). If none is given, the
	 * Matrix's type is used.*/
	public void normalize(T=Type)()
	{
		final T normal=this.determinant!(T)();
		foreach(ref value; this) value/=normal;
	}
	
	/**Inverts this Matrix.
	 * Instanciate this function with a type to determine the resolution of the invertion.
	 * Throws: MatrixException if the determinant of the current matrix is 0 (matrix is not invertible).$(BR)
	 * MatrixSizeException if the matrix is not square (the determinant cannot be calculated)
	 * See_Also: getInverse()*/
	public void invert(T=Type)()
	{
		final T det=this.determinant!(T)();
		version(Phobos) 
			if (approxEqual(det, 0)) throw new MatrixException("Cannot invert matrix: it's determinant is 0");
			else {}
		else static assert(false, "You must specify which library you are using (-version=Phobos)");
		Type[][] newData=new Type[][](this.m(), this.n()); //generate new data
		scope(failure) delete newData;
		
		//2x2 matrix
		if (this.m()==2)
		{
			newData[0][0]=this[1,1];
			newData[0][1]=-this[0,1];
			newData[1][0]=-this[1,0];
			newData[1][1]=this[0,0];
			delete data;
			data=newData;
			this/=det; //divide current matrix by the determinant to obtain the inverse
			return;
		}
		
		bool cofactor=0;
		for (auto i=0; i<this.m(); i++)
			for (auto j=0; j<this.n(); j++)
			{
				if (cofactor) newData[j][i]=-this.minorMatrix(i,j).determinant!(T)();
				else newData[j][i]=this.minorMatrix(i,j).determinant!(T)();
				cofactor=!cofactor;
			}
		delete data;
		data=newData;
		this/=det; //divide current matrix by the determinant to obtain the inverse
		return;
	}
	
	/**Returns: this Matrix's inverted Matrix instantiated to the same type as this Matrix
	 * Note: Uses the .invert!(T)() routine.
	 * Throws: Same as invert()
	 * See_Also: invert(), dup!(T)*/
	public const Matrix!(Type) getInverse(T=Type)()
	{
		return (this.dup().invert!(T)());
	}
	
	/* *********************** */
	/* ** MATRIX PROPERTIES ** */
	/* *********************** */
	
	/**Returns: the characteristic polynom of this Matrix on a vector of the type specified.
	 * If none is given, the Matrix's type is used.
	 * Note: The returned vector contains the constant in decreasing degree:
	 * [a*t^(degree()), bt^(degree-1), ..., zt^(0)], so the number of factors
	 * is equal to the degree of the Matrix.$(BR)
	 * This method is faster if the Matrix's degree==2 in which case, the result is
	 * [1, -trace(), determinant()].
	 * Throws: MatrixSizeException if this Matrix is not square. */
	public const T[] getCharPoly(T=Type)()
	{
		if (!isSquare()) throw new MatrixSizeException("Cannot obtain the characteristic polynom of a non-square matrix...");
		if (this.degree()==2) return [1, -this.trace!(T)(), this.determinant!(T)()];
		else assert(false, "Not yet implemented...");
	}
	
	/**Returns: true if this Matrix is singular (determinant equals 0)
	 * Note: Singular Matrices cannot be inverted.$(BR)
	 * The determinant is compared with std.numeric.approxEqual to cover for rounding errors
	 * in case of floating point matrices.
	 * Throws: MatrixSizeException if this Matrix is not square.*/
	bool isSingular()
	{
		version (Phobos)
			if (approxEqual(this.determinant(), 0)) return true;
			else return false;
		else static assert(false, "You must specify which library you are using (-version=Phobos)");
	}
	
	///Returns: The degree of this Matrix or 0 if the Matrix is not square
	public const size_t degree()
	{
		if (this.m()==this.n()) return this.m();
		else return 0;
	}
	
	///If degree returns 0 (false) the Matrix is not square
	alias degree isSquare;
	
	///Returns: the order of this Matrix (mxn)
	public const size_t order()
	{
		return (this.m()*this.n());
	}
	
	/**Returns: true if this Matrix is symmetric: it's elements are mirrored along the main diagonal.
	 * Throws: MatrixSizeException if this matrix is not square (non-transposable).*/
	public const bool isSymmetric()
	{
		if (this==this.getTranspose()) return true; 
		else return false;
	}
	
	/+
	
	Funny bug: this function is complete, but fails to compile. DMD output follows:
	----------------------
	matrix.d(587): Error: expression (negative.opNeg)() is void and has no value
	matrix.d(587): Error: cannot implicitly convert expression ((negative.opNeg)()) of type void to matrix.Matrix!(int).Matrix
	--------------------                                                       ^
	If you can't find the bug, just notice this slight typo:                   | (the parentesis)
	
	/**Returns: true if this Matrix is skew-symmetric: it's elemets are mirrored and negated
	 * along the main diagonal. Also, it's main diagonal must be composed of 0.
	 * Throws: MatrixSizeException if this matrix is not square (non-transposable).*/
	public const bool isSkewSymmetric()
	{
		auto scope negative=this.getTranspose();
		negative=-negative;
		if (this!=negative) return false; //replace negative with "this.getTranspose()"
		foreach(element; this.getDiagonal())
			version(Phobos) {if (!approxEqual(element, 0)) return false;}
			else static assert(false, "You must specify which library you are using (-version=Phobos)");
	}
	+/
	
	/**Returns: True if this Matrix is Hermitian: it's elements mirrored along the main diagonal
	 * are each others conjugate.
	 * Throws: MatrixSizeException if this Matrix is not square.
	 * Note: Only Matrices instanciated to a complex type can be Hermitian.*/
	public const bool isHermitian()
	{
		//wouldn't it be great if I could know if Type was imaginary? Sort of like __traits(isImaginary)...
		//assume this.Type is imaginary
		static if(!is(Type:creal)) throw new MatrixException("Only complex type matrices may be Hermitian.");
		else
		{
			if (!isSquare()) throw new MatrixSizeException("Only square matrices may be Hermitian");
			for(auto r=0; r<this.m(); r++)
				for (auto c=r; c<this.n(); c++) //only compare top triangle
					if (c==r) continue; //we don't care about the main diagonal
					else version (Phobos) {if (!approxEqual(this[r,c], conj(this[c,r]))) return false;}
						else static assert(false, "You must specify which library you are using (-version=Phobos)");
			return true;
		}
	}
	
	/**Returns: True if this Matrix is Toeplitz: the elements on it's diagonal are all equal.
	 *
	 * You can specify a value to test against, as soon as DMD allows it.
	 * Throws: MatrixSizeException if this Matrix is not square.*/
	public const bool isToeplitz(/*in Type value=this[0,0]*/)
	{
		final value=this[0,0]; //remove this line when above is valid
		if (!isSquare()) throw new MatrixSizeException("Only square matrices may be Toeplitz");
		version(Phobos)
		{
			foreach(element; this.getDiagonal()) if (!approxEqual(element, value)) return false;
		}
		else static assert(false, "You must specify which library you are using (-version=Phobos)");
		return true;
	}
	
	/**Returns: True if this Matrix is idempotent: it's square is equal to itself:
	 * "matrix*matrix==matrix"
	 * Throws: MatrixSizeException if this Matrix is not square.*/
	public const bool isIdempotent()
	{
		if (!isSquare()) throw new MatrixSizeException("Only square matrices may be Idempotent");
		//Bug: this should be "this*this==this", however, it fails to compile giving an error on opMul
		if (/*this**/this==this) return true;
		else return false;
	}
	
	/* *************** */
	/* ** OPERATORS ** */
	/* *************** */

	/**Returns: value at Matrix position [row, col].
	 * Throws: ArrayOutOfBounds if the requested element is not valid.*/
	public const Type opIndex(in size_t row, in size_t col)
	{
		return data[row][col]; //D will automatically perform Bounds Check ;)
	}
	
	/**Returns: this Matrix's _row row. This allows matrix[x][y] operations to be valid,
	 * although for this case, matrix[x,y] should be used for clarity.
	 * Throws: ArrayOutOfBounds if the requested element is not valid.
	 * See_Also: opIndex() (overload)*/
	public Type[] opIndex(in size_t row)
	{
		return data[row];
	}

	/**Assigns value to Matrix element [row, col].
	 * Throws: ArrayOutOfBounds if required coordinates are too big.*/
	public void opIndexAssign(in Type value, in size_t row, in size_t col)
	{
		data[row][col]=value; //D will automatically performs Bounds Check ;)
	}
	
	/**Assigns vector to Matrix position [row].
	 * Note: the data _vector is not duplicated. This method will not modify _vector, but
	 * subsquent calls to non-const funtions of this matrix may modify it. Deleting vector
	 * will invalidate the Matrix's _row.
	 * Throws: ArrayOutOfBounds if the requested element is not valid.
	 * MatrixSizeException if the vectors' size is not conformant with this Matrix's n*/
	public void opIndexAssign(Type[] vector, in size_t row)
	in
	{
		assert(vector !is null, "Matrix: vector passed to opIndexAssign is null");
	}
	body
	{
		if (vector.length!=this.n()) throw new MatrixSizeException("Vector size is not equal to Matrix's number of columns");
		data[row]=vector; //D will automatically performs Bounds Check ;)
	}

	/**Checks two matrices for equality.
	 * Note: this function uses std.numeric.approxEqual() to compare floating point numbers
	 * Throws: MatrixSizeException if matrices have different sizes.*/
	public const bool opEquals(in Matrix matrix)
	in
	{
		assert(matrix !is null, "Matrix: Matrix passed to opEquals is null");
	}
	body
	{
		if ((matrix.m()!=this.m())||(matrix.n()!=this.n()))
			throw new MatrixSizeException("Can't compare matrices of different sizes.");
		//note: a for loop is used here instead of a foreach because the comparing matrix may be derivated,
		//and hence it's data[][] may be organized differently.
		for(auto i=0; i<this.m(); i++)
			for(auto j=0; j<this.n(); j++)
				version(Phobos) {if (!approxEqual(matrix[i,j], this[i,j])) return false;}
				else static assert(false, "You must specify which library you are using (-version=Phobos)");
		return true;
	}

	///Divides all elements of this Matrix by a scalar. You cannot divide a Matrix by a Matrix.
	public const Matrix!(Type) opDiv(T)(in T divisor)
	{
		auto ret=new Matrix!(Type)(m, n);
		foreach(i, Type[] row; ret.data)
			foreach(j, inout Type value; row)
				value=cast(Type)(this[i,j]/divisor);
		return ret;
	}

	///ditto
	public void opDivAssign(T)(in T divisor)
	{
	    for(auto i=0; i<m; i++)
            for (auto j=0; j<n; j++)
                this[i, j]=cast(Type)(this[i, j]/divisor);
	}

	/**Multiplies this matrix by multiplier, being it a scalar or another Matrix.
	 *
	 * Note: You can only multiply matrices which inner size is equal (axb and bxc).
	 * The resulting Matrix will have outer dimensions (axc).
	 * Matrix multiplication is NOT commutative.
	 *
	 * Throws: Matrix size exception if inner matrix sizes are not equal*/	
	public const Matrix!(Type) opMul(T)(in T multiplier)
	{//bug: this stopped working... why wouldn't it be valid?
		static if( is(T : Matrix) ) //matrix multiplication
		{
			if (this.n()!=multiplier.m()) throw new MatrixSizeException("To multiply two matrices of sizes 'a x b' and 'c x d', 'c' must equal 'b'");
			auto ret=new Matrix!(Type)(this.m(), multiplier.n()); //create a matrix to be returned
			scope(failure) delete ret; //delete created matrix on exception throw
			for (auto i=0; i<ret.m(); i++)
				for (auto j=0; j<ret.n(); j++)
				{
					ret[i,j]=0; //init the value. It will equal the sum of A[i,k]*B[k,j], k<n
					for (auto k=0; k<n; k++)
						ret[i,j]=ret[i,j]+cast(Type)(this[i,k]*multiplier[k,j]); //ret[i,j]+= fails to compile
				}
			return ret;
		}
		else //scalar multiplication
		{
			auto ret=this.dup(); //create a matrix to be returned
			scope(failure) delete ret; //delete created matrix on exception throw
			for (auto i=0; i<ret.m(); i++)
				for (auto j=0; j<ret.n(); j++)
					ret[i,j]=ret[i,j]*multiplier;
			return ret;
		}
	}

	///ditto
	public void opMulAssign(T)(in T multiplier)
	{
		static if (is(T : Matrix)) //Matrix multiplication
		{
			if (this.n()!=multiplier.m())
				throw new MatrixSizeException("To multiply two matrices of sizes 'a x b' and 'c x d', 'c' must equal 'b'");
			Type[][] newData=new Type[][](this.m(), multiplier.n()); //generate new data arrays
			scope(failure) delete newData; //failed some calculation, preserve current data
			foreach(i, Type[] row; newData)
				foreach(j, inout Type value; row)
				{
					value=0; //init the value. It will equal the sum of A[i,k]*B[k,j], k<n
					for (auto k=0; k<n; k++) //perform the vectorial multiplication
						value+=this[i,k]*multiplier[k,j];
				}
			delete data;
			data=newData; //switch to new data
			return;
		}
		else //scalar multiplication
			for(auto i=0; this.m(); i++)
                for (auto j=0; j<this.n(); j++)
					this[i,j]=this[i,j]*multiplier; //this[i,j]*=multiplier; -> compiler error
	}

	/**Adds two Matrices.
	 * Throws: MatrixSizeException if Matrices are not of the same size. */
	public const Matrix!(Type) opAdd(in Matrix adder)
	{
		if ((this.m()!=adder.m())||(this.n()!=adder.n()))
			throw new MatrixSizeException("Cannot sum two matrices of different sizes");
		auto ret=this.dup();
		scope(failure) delete ret;
		for (auto i=0; i<this.m(); i++)
			for (auto j=0; j<this.n(); j++)
				ret[i,j]=cast(Type)(ret[i,j]+adder[i,j]);
		return ret;
	}

	///ditto
	public void opAddAssign(in Matrix adder)
	{
		if ((this.m()!=adder.m())||(this.n()!=adder.n()))
			throw new MatrixSizeException("Cannot sum two matrices of different sizes");
		for (auto i=0; i<this.m(); i++)
			for (auto j=0; j<this.n(); j++)
				this[i,j]=cast(Type)(this[i,j]+adder[i,j]); //this+= fails to compile
	}

	/**Subtracts two Matrices.
	 * Throws: MatrixSizeException if Matrices are of unmatching size.*/
	public const Matrix!(Type) opSub(in Matrix subtractor)
	{
		if ((this.m()!=subtractor.m())||(this.n()!=subtractor.n()))
			throw new MatrixSizeException("Cannot subtract two matrices of different sizes");
		auto ret=new Matrix!(Type)(m, n);
		scope(failure) delete ret;
		for (auto i=0; i<ret.m(); i++)
			for (auto j=0; j<ret.n(); j++)
				ret[i,j]=cast(Type)(this[i,j]-subtractor[i,j]);
		return ret;
	}

	///ditto
	public void opSubAssign(in Matrix subtractor)
	{
		if ((this.m()!=subtractor.m())||(this.n()!=subtractor.n()))
			throw new MatrixSizeException("Cannot sum two matrices of different sizes");
		for (auto i=0; i<this.m(); i++)
			for (auto j=0; j<this.n(); j++)
				this[i,j]=cast(Type)(this[i,j]-(subtractor[i,j]));
	}
	
	/**Negates all values on this Matrix
	 * This operation may have unexpected results when the current Matrix is
	 * instanciated to an unsigned type or if the type does not have the
	 * opNeg function implemented.*/
	public void opNeg()
	{
		foreach(ref value; this) value=-value;
	}
	
	/**Adds columns to this Matrix (Shift Left Operator).
	 *
	 * If you provide a single value, that value is repeated for every
	 * row of the new column. If you provide an array, that same array is
	 * interpreted as (a) new column(s), depending on it's size. If you provide 
	 * another Matrix, that Matrix is joined with the current one.
	 *
	 * Throws: MatrixSizeException if the size of the vector passed is not a
	 * multiple of the number of rows on this Matrix (this.m()) or the number
	 * of rows of the adding Matrix is not the same.
	 * Values are converted using std.conv.to!(Type)() which may throw
	 * convertion errors.
	 * 
	 * Note: all values of the incoming vector/matrix are cast to the type
	 * of the current Matrix using std.conv.to!().
	 *
	 * Bugs: if a vector is provided, it is assumed not to be null.
	 *
	 * See_Also: dup!(T)()*/
	public Matrix!(Type) opShl(T)(T value)
	{
		auto ret=this.dup;
		scope(failure) delete ret;
		ret<<=value;
		return ret;
	}
	
	/+
	DMD 2.007 is buggy and does not allow the following specialization, although it is documented
	on www.digitalmars.com/d/templates-revisited.html :
	class Foo(
	(snip)
	C:B,          // C must be of class B or derived
					// from B
	(snip)
	)
	
	NOTE TO SELF: If this bug ever gets killed, fix opMul as well.
	Also, see if "opShl(T:T[])(T value)" or "opShl(T)(T[] value)" becomes legal/unambigous
	
	public Matrix opShl(T:Matrix)(T matrix)
	{
		return null;
	}+/
	
	///ditto
	public void opShlAssign(T)(T value)
	{
		version (Phobos)
		{
			static if (isArray!(T))
			{
				final auto remain=value.length%this.m();
				if (remain!=0)
					throw new MatrixSizeException("Cannot add columns. The size of the provided vector is incompatible");
				//TODO: refine this for loop to prevent reallocations
				for (auto i=0; i<value.length; i+=this.m())
					foreach(r, ref row; data) row~=to!(Type)(value[r]);
				return;
			}
			else
			{
				foreach(ref row; data) row~=to!(Type)(value);
				return;
			}
		}
		else static assert (false, "You must specify which library you are using (-version=Phobos)");
	}
	
	/**Removes num columns from this Matrix (Shift Right Operator)
	 *
	 * Throws: MatrixSizeException if you try to remove all columns from this Matrix, or more than
	 * those available. */
	public Matrix opShr(in size_t num)
	{
		auto ret=this.dup();
		scope(failure) delete ret;
		ret>>=num;
		return ret;
	}
	
	///ditto
	public void opShrAssign(in size_t num)
	{
		if (num==0) return;
		if (num>=this.n()) throw new MatrixSizeException("Failed to remove columns: the Matrix is too small");
		foreach(ref row; data) row=row[0..($-1-num)]; //you gotta love D for this trick!
	}
	
	/**Adds a value, array or Matrix to the current Matrix as rows. If types differ, they
	 * will be converted to the current Matrix's type
	 *
	 * Throws: MatrixSizeException if the size of the vector passed is not
	 * multiple of the number of columns on this Matrix (this.n()) or the number
	 * of columns of the adding Matrix is not the same. $(BR)
	 * Values are converted using std.to!() which may throw convertion errors.*/
	public Matrix opCat(T)(T value)
	{
		static if( is(T : Matrix) ) //matrix concactenation
		{
			writeln("adding another matrix");
			if (this.n()!=value.n()) throw new MatrixSizeException("Cannot add matrices: unmatching size");
			auto ret=new Matrix!(Type)(this.m(),this.n()+value.n()); //create a matrix to be returned
			scope(failure) delete ret; //delete created matrix on exception throw
			for (auto i=0; i<this.m(); i++)
				for (auto j=0; j<this.n(); j++)
					ret[i,j]=this[i,j]; //copy data from current matrix
			for (auto i=0; i<value.m(); i++)
				for (auto j=value.n(), z=0; j<ret.n(); j++, z++)
					ret[i,j]=value[i,z]; //copy data from the other matrix
			return ret; //return the new
		}
		
		version (Phobos)
			static if (isArray!(T)) auto ret=this.dup!(typeof(value[0])); //adding an array
			else auto ret=this.dup!(T);
		else static assert (false, "You must specify which library you are using (-version=Phobos)");
		scope(failure) delete ret;
		ret~=value;
		return ret;
	}
	
	///ditto
	public void opCatAssign(T)(T value)
	{
		version (Phobos)
		{
			static if (isArray!(T))
			{
				final auto remain=value.length%this.n();
				if (remain!=0) throw new MatrixSizeException("Cannot add rows. The size of the provided vector is incompatible");
				for (auto i=0; i<value.length; i+=this.n())
					data~=to!(Type[])(value[i..(i+this.n())]);
				return;
			}
			else
			{
				data.length=data.length+1;
				data[$-1].length=this.n();
				foreach(ref element; data[$-1]) element=to!(Type)(value);
				return;
			}
		}
		else static assert (false, "You must specify which library you are using (-version=Phobos)");
	}

	/**Checks if value is inside the Matrix (use: 'Matrix in value')
	 * Examples:
	 * --------------------
	 *	auto matrix=new Matrix!(int)(2, 2, 3); //creates a 2x2 matrix filled with number '3'
	 *	if (matrix in 3) writeln("matrix has number 3"); //this will evaluate to true
	 *	if (matrix in 0) writeln("matrix has number 0"); //This will evaluate to false
	 * --------------------
	 * Output: 'matrix has number 3'*/
	public const bool opIn(in Type value)
	{
		for (auto i=0; i<this.m; i++)
			for (auto j=0; j<this.n; j++)
				if (this[i,j]==value) return true;
		return false;
	}

	/**Implements the foreach and foreach_reverse behaviour. Cycles through all values on the Matrix
	 * Examples:
	 * -------------------
	 * foreach (ref value; matrix) value=5; //this will set all value of the matrix to 5
	 * int sum;
	 * foreach (value; matrix) sum+=value; //obtain the sum of all values on the matrix
	 * foreach (ref value; matrix) foo(value); //calls "foo" for every value of the function
	 * foreach_reverse (value; matrix) write(value); //prints all values of the matrix in reverse order
	 * -------------------*/
	int opApply(int delegate(ref Type) func)
	{
		int result;
		foreach(row; data)
			foreach(ref value; row)
			{
				result=func(value);
				if (result) return result;
			}
		return result;
	}
	
	///ditto
	int opApplyReverse(int delegate(ref Type) func)
	{
		int result;
		foreach_reverse(row; data)
			foreach_reverse(ref value; row)
			{
				result=func(value);
				if (result) return result;
			}
		return result;
	}
	
	/**Return: the Type of this Matrix's data.
	 * Examples:
	 * -----------------------
	 * auto matrix=new Matrix!(int)(4, 4);
	 * matrix.getType foo; //foo is declared as "int"
	 * -----------------------
	 */
	public template getType()
	{
		alias Type getType;
	}

	///Returns: this Matrix's number of rows
	public const size_t m()
	{
		return data.length;
	}

	///Returns: this Matrix's number of columns
	public const size_t n()
	{
		return data[0].length;
	}

	///Destroys this Matrix. It's data arrays are also explicitly deleted.
	~this()
	{
		delete data;
	}

	///Data arrays for this Matrix
	protected Type[][] data;
}

//version(unittest&&Phobos) import std.random; //yeah yeah, version(unittest) what was I thinking...
version(Phobos) import std.random;
unittest
{
	//version(Phobos import std.random; //scope import don't work. What was I thinking?
	//To be completed
	assert(Matrix!(int).identity(4).determinant()==1, "Matrix: Failed determinant unittest");
}

//test program

import std.stdio;
import std.traits;
import std.random:rand;

ubyte fa()
{
	return cast(ubyte)(rand%10);
}

void main ()
{
	auto test=new Matrix!(int)(3, 3, 3);
	//writeln(test.toString("\n"));
	short[] vector=[2, 2, 2];
	
	//bug: if the following line is not commented, DMD will stop without giving any reason
	//test=test<<vector;
	//use the "-v" switch to see that it breaks on "semantic3"
	
	auto outra=new Matrix!(ubyte)(10,11,&fa);
	//auto outra=uma.dup!(real)();
	writeln(outra);
	writeln(outra.solve!(real)());
}
